import os


def rc_file():
    """ Return the full .vacationrc path for convenience. """
    return os.path.join(os.path.expanduser('~'), '.vacationrc')


def touch_rc():
    """ Create a .vacationrc file if none exists. """
    if not os.path.isfile(rc_file()):
        open(rc_file(), 'a').close()
        print('Created file: {}'.format(rc_file()))



def read_rc():
    """ Read file and return entries as a list. """
    try:
        with open(rc_file(), 'r') as rc:
            return rc.readlines()
    except IOError:
        print('Error reading your ~/.vacationrc file!')
        return []


def append_rc(entry):
    """ Append either a list of strings or a string to our file. """
    if not entry:
        return
    try:
        with open(rc_file(), 'a') as f:
            if isinstance(entry, list):
                f.writelines(entry)
            else:
                f.write(entry + '\n')
    except IOError:
        print('Error writing your ~/.vacationrc file!')



"""
Sample vacation transaction file:

2015-11-08: days 10.5
2015-11-08: rate 1.333
2015-12-25: off
2015-12-26: off
2015-01-01: rate 1.5
2015-01-31: off

"""
import datetime

import holidays
import workdays


def validate_setup(transactions):
    """ First two transactions must set rate & days. """
    if not transactions:
        return True
    try:
        first, second = transactions[:2]
    except ValueError:
        print('Error: vacationrc file must have both initial days and rates entries')
        return False

    parts1, parts2 = first.split(), second.split()

    if parts1[0] != parts2[0]:
        print('Error: First two entries in vacationrc must have the same date')
        return False  # Dates must match

    if 'rate' not in (parts1[1], parts2[1]) or 'days' not in (parts1[1], parts2[1]):
        print('Error: First two entries in vacationrc must set days and rate')
        return False

    return True


def _parse_transaction_entry(entry):
    """ Validate & parse a transaction into (date, action, value) tuple. """
    parts = entry.split()

    date_string = parts[0]
    try:
        date = datetime.datetime.strptime(date_string[:-1], '%Y-%m-%d').date()
    except ValueError:
        raise ValueError('Invalid date in vacationrc for entry: {}'.format(entry))

    if len(parts) < 2:
        raise ValueError('.vacationrc missing an action for entry: {}'.format(entry))
    action = parts[1].lower()
    if action not in ('days', 'rate', 'off', 'adjust', 'show'):
        raise ValueError('Invalid action in vacationrc for entry: {}'.format(entry))

    try:
        value = float(parts[2])
    except IndexError:
        value = None
    except (ValueError, TypeError):
        raise ValueError('Invalid value in vacationrc for entry: {}'.format(entry))

    return (date, action, value)


def stat_holidays(province='BC', year=2015):
    """ Returns a list of holiday dates for a province and year. """
    return holidays.Canada(state=province, years=year).keys()


def sum_transactions(transactions):
    """ Sums transactions into a total of remaining vacation days. """
    workdays_per_year = 250
    last_date = None
    rate = 0
    day_sum = 0
    transactions.append('{}: show'.format(datetime.date.today().strftime('%Y-%m-%d')))
    for transaction in transactions:
        date, action, value = _parse_transaction_entry(transaction)

        if action == 'days':
            day_sum = value
        elif action == 'rate':
            rate = float(value) / workdays_per_year

        if last_date is None:
            last_date = date

        elapsed = workdays.networkdays(last_date, date, stat_holidays()) - 1
        if action == 'off':
            elapsed -= 1  # Didn't work that day
            day_sum -= 1  # And we used a day

        day_sum += rate * elapsed
        last_date = date

    return round(day_sum, 1)




from __future__ import absolute_import

import argparse
import datetime

import lexer
import transactions
import rc


def setup_args():
    argp = argparse.ArgumentParser('Vacation Tracker')
    argp.add_argument('input', nargs='*', help='TODO')  # TODO: Write help text
    return argp.parse_args()


def main():
    args = setup_args()

    rc.touch_rc()  # Make sure ~/.vacationrc exists

    tokens = lexer.lex(args.input)
    for token in tokens:
        action = token[0]
        if action == 'show':
            break
        elif action == 'take':
            date_str = token[1] + '-{}'.format(datetime.date.today().year)
            date = datetime.datetime.strptime(date_str, '%b %d-%Y').date()
            rc.append_rc('{}: off'.format(date.strftime('%Y-%m-%d')))
        elif action == 'setrate':
            date = datetime.date.today()
            rc.append_rc('{}: rate {}'.format(date.strftime('%Y-%m-%d'), token[1]))
        elif action == 'setdays':
            date = datetime.date.today()
            rc.append_rc('{}: days {}'.format(date.strftime('%Y-%m-%d'), token[1]))

    trans = rc.read_rc()  # Read transactions
    if not trans:
        print('Your .vacationrc file is empty! Set days and rate.')
    else:
        if transactions.validate_setup(trans):  # Validate
            days_remaining = transactions.sum_transactions(trans)  # sum up our new days remaining
            print('{} vacation days remaining'.format(days_remaining))


if __name__ == '__main__':
    main()



SHOW = 'show'
TAKE = 'take'
SET = 'set'
RATE = 'rate'
DAYS = 'days'
TAKERANGE = 'takerange'
SETRATE = 'setrate'
SETDAYS = 'setdays'
MONTHS = ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec']

def lex(args):
    """ Lex input. """
    if len(args) == 0:
        return [(SHOW,)]
    elif args[0] == SHOW:
        return [(SHOW,)]
    elif args[0] == SET and args[1] == RATE:
        return tokenizeSetRate(args[2:])
    elif args[0] == SET and args[1] == DAYS:
        return tokenizeSetDays(args[2:])
    elif isMonth(args[0]):
        return tokenizeTake(args)

def tokenizeSetRate(args):
    if not args[0:]:
        raise ValueError('Missing args for <set rate>')
    try:
        rate = float(args[0])
    except ValueError:
        raise ValueError('Invalid rate: {}'.format(args))
    return [(SETRATE, '{}'.format(rate))]

def tokenizeSetDays(args):
    if not args[0:]:
        raise ValueError('Missing args for <set days>')
    try:
        days = float(args[0])
    except ValueError:
        raise ValueError('Invalid number of days: {}'.format(args))
    return [(SETDAYS, '{}'.format(days))]

def tokenizeTake(args):
    ret = [(TAKE, date) for date in lexDate(args)]
    return ret

def isMonth(arg):
    month = arg[:3].lower()
    return month in MONTHS

def lexDate(args):
    month = args[0][:3].lower()
    if not isMonth(args[0]):
        raise ValueError('Not a valid month')
    dates = []
    for arg in args[1:]:
        day = getDay(arg)
        dates.append('{} {}'.format(month, day))
    return dates

def getDay(arg):
    arg = arg.strip(',')
    try:
        return int(arg)
    except ValueError:
        raise ValueError('Invalid day: {}'.format(arg))



from setuptools import setup


def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='vacation',
      version='0.4.6',
      description='Track your vacation days.',
      long_description=readme(),
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Topic :: Utilities',
      ],
      url='https://github.com/pulseenergy/vacation',
      author='Enernoc',
      author_email='supermitch@gmail.com',
      license='GPL2',
      packages=['vacation'],
      include_package_data=True,
      install_requires=[
          'holidays',
          'workdays',
      ],
      test_suite='nose.collector',
      tests_require=['nose'],
      zip_safe=False,
      entry_points={
          'console_scripts': ['vacation=vacation.vacation:main'],
      },)
